;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Read in data:
;; 
;; The first line is a row of labels: the rest is data.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def a (read-matrix-data "stuff.txt"))
(def g (read-matrix-data "group_centroids_UTM17.txt"))
(def p (read-matrix-data "park_centroids_UTM17.txt"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                Establish new variables from the data matrices:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(mvsq 
 '("ORIGIN" "DESTINATION" "DISTANCE")
 (column-list (but-rows a (iseq 1)))
 )
(mvsq 
 '("park_x" "park_y" "park_id")
 (column-list (but-rows p (iseq 1)))
 )
(mvsq 
 '("group_x" "group_y" "group_id")
 (column-list (but-rows g (iseq 1)))
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                   Change from arrays to lists:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setq origin (coerce origin 'list)
      destination (coerce destination 'list)
      DISTANCE (coerce DISTANCE 'list)
      park_x (coerce park_x 'list)
      park_y (coerce park_y 'list)
      group_x (coerce group_x 'list)
      group_y (coerce group_y 'list)
      done nil
      )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;             Determine how many parks and groups from script:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def parks (remove-duplicates origin))
(def groups (remove-duplicates destination))
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                Compare to lengths of data:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(if (or
     (/= (length parks) (length park_x))
     (/= (length groups) (length group_x))
     )
    (let ()
      (print "Error: dimensions don't agree!")
      (break)
      )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Define matrices
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def
 network-distance-matrix
 (make-array
  (list 
   (length parks)
   (length groups)
   )
  )
 )
(def
 euclidean-distance-matrix
 (make-array
  (list 
   (length parks)
   (length groups)
   )
  )
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                Establish distance matrices
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(let ()
  (mapcar
   (lambda(x y z)
     (setf
      (aref network-distance-matrix x y)
      z
      )
     )
   (- origin 1)
   (- destination 1)
   distance
   )
  nil
  )

(let (
      (parks (transpose (list park_x park_y)))
      (groups (transpose (list group_x group_y)))
      )
  (mapcar
   (lambda(x y)
     (setf
      (aref euclidean-distance-matrix x y)
      (norm (- (elt parks x) (elt groups y)))
      )
     )
   (- origin 1)
   (- destination 1)
   )
  nil
  )

#|
(- 
 (row 1 euclidean-distance-matrix) 
 (row 1 network-distance-matrix))
|#


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    Read Park and Group characteristics
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def pchar (read-matrix-data "park_data.txt"))
(mvsq 
 (row 1 pchar)
 (column-list (but-rows pchar (iseq 1)))
 )
(setq
 ID (coerce id 'list)
 PICNIC (coerce picnic 'list)
 WALK (coerce walk 'list)
 GYMS (coerce gyms 'list)
 INSW (coerce insw 'list)
 OUSW (coerce ousw 'list)
 PLAY (coerce play 'list)
 SOCC (coerce socc 'list)
 BASE (coerce base 'list)
 FOOT (coerce foot 'list)
 OUBB (coerce oubb 'list)
 OUVB (coerce ouvb 'list)
 TENN (coerce tenn 'list)
 )

(setf
 park_vals
 (mapcar
  (lambda(ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN)
    (if (and ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN)
	(+ ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN)
      nil
      )
    )
  ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN
  )
 ;; now again, using the minimum value of park_vals for those with missing
 ;; data:
 park_vals
 (mapcar
  (lambda(ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN)
    (if (and ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN)
	(+ ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN)
      (min (select park_vals (which park_vals)))
      )
    )
  ID PICNIC WALK GYMS INSW OUSW PLAY SOCC BASE FOOT OUBB OUVB TENN
  )
 )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    Define a null distance (if two centroids are right on top
;;                    of one another, distance between is not really 0!):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def null-dist 100)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       Here are the results:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setf
 euclidean-values
 (mapcar
  (lambda(x) (sum (/ park_vals (+ x null-dist))))
  (column-lists euclidean-distance-matrix)
  )
 network-values
 (mapcar
  (lambda(x) (sum (/ park_vals (+ x null-dist))))
  (column-lists network-distance-matrix)
  )
 done nil
 )

(def results
     (bind-columns
      (coerce group_id 'list)
      euclidean-values
      network-values
      )
     )
(mprint results)

(lines off)
(plot euclidean-values  network-values)
(write-matrix-to-file 
 results
 "results.txt"
 :header "ID,euclidean,network~%"
 :fmt "\"~a\",~f,~f~%")